/*
* dave.myQuestAPI.{..}
* getItemHand
* getItems
* getFormTitle
* getFromDesc
* getFromButton
*/

import {
    system,
    world
} from "@minecraft/server";
import {
    ActionFormData,
    ModalFormData
} from '@minecraft/server-ui'

// Used to import variables from config.js
import * as dave from './config.js';

const {
  system: { enable: systemEnable, tag: systemTag, inactive: systemInactive },
  getIdentifier: { author: getAuthor, title: getTitle, description: getDescription },
  getQuest: { itemHand: getItemHand, items: getItems, rewards: getRewards },
  getForm: {
    title: { enable: getTitleEnable, title: getTitleTitle },
    description: { description: getDescriptionDescription, complated: getDescriptionComplated, notComplated: getDescriptionNotComplated },
    icon: { enable: getIconEnable, confirm: getIconConfirm, items: getIconItems },
    button: { back: getButtonBack, check: getButtonCheck, about: getButtonAbout }
  }
} = dave.myQuestAPI;

world.beforeEvents.itemUse.subscribe((eventData) => {
  const { itemStack: item, source: player } = eventData;

  if (item.typeId === getItemHand) {
    if (systemEnable || player.hasTag(systemTag)) {
      system.run(() => {
        formMain(player);
      });
    } else {
      const runCmd = player.runCommandAsync;
      runCmd(`tellraw @s {"rawtext":[{"text":"${systemInactive}\n\n "}]}`);
      runCmd(`tellraw @s {"rawtext":[{"text":"===== ${getTitle} =====\n§e已得到myQuestAPI使用许可: §r${systemEnable}\n§e版本: 1.1.0§r${systemVersion}\n§e模组作者:皓星辰 §r@${getAuthor}\n§e源代码来自: §r@abcdave"}]}`);
    }
  }
});

const formMain = (player) => {
  const form = new ActionFormData().title(getTitle).button(getButtonAbout);
  const LIST = [];
  let COUNT = 0;
  
  for (const thisItems of getItems) {
    const getTitleAuto = thisItems.split(/:(.*)/s)[1].replaceAll('_', ' ').toLowerCase().replace(/(^\w{1})|(\s+\w{1})/g, letter => letter.toUpperCase());
    const hasTag = player.hasTag(thisItems);

    const title = getTitleEnable ? getTitleAuto : getTitleTitle[COUNT];
    const icon = getIconEnable ? (hasTag ? getIconConfirm : `textures/${getIconItems[COUNT]}`) : undefined;
    const button = hasTag ? title + "\n §2任务完成！" : title;

    form.button(button, icon);
    LIST.push(COUNT);
    COUNT++;
  }

  form.show(player).then(response => {
    if (response.selection === 0) {
      formAbout(player);
    }
    if (response.selection) {
      formSelection(player, response.selection);
    }
  });
};

const formSelection = (player, string) => {
  const getItemName = getItems[string - 1].split(/:(.*)/s)[1].replaceAll('_', ' ').toLowerCase().replace(/(^\w{1})|(\s+\w{1})/g, letter => letter.toUpperCase());
  const getRewardName = getRewards[string - 1][0].split(/:(.*)/s)[1].replaceAll('_', ' ').toLowerCase().replace(/(^\w{1})|(\s+\w{1})/g, letter => letter.toUpperCase());
  const getIsComplate = player.hasTag(getItems[string - 1]);
  const form2 = new ActionFormData().title(getTitle);

  if (getIsComplate) {
    form2.body(`${getDescriptionDescription[string - 1]}\n\n§a当你物品栏中拥有目标物品时，可提交任务\n§e提交任务不消耗任何物品\n§e进度: §a已完成`);
  } else {
    form2.body(`${getDescriptionDescription[string - 1]}\n\n§a当你物品栏中拥有目标物品时，可提交任务\n§e提交任务不消耗任何物品\n§e进度: §c未完成`);
  }

  form2.button(getButtonCheck).button(getButtonBack);
  form2.show(player).then(response2 => {
    if (!response2.isCanceled) {
      if (response2.selection === 0 && player.hasTag(getItems[string - 1])) {
        player.runCommandAsync(`title @s title §c§l你已经完成这个任务了`);
      }
      if (response2.selection === 0 && !player.hasTag(getItems[string - 1])) {
        for (let i = 0; i < 36; i++) {
          const inventoryItem = player.getComponent("inventory").container.getItem(i);
          if (inventoryItem && inventoryItem.typeId === getItems[string - 1]) {
            player.runCommandAsync(`give @s ${getRewards[string - 1][0]} ${getRewards[string - 1][1]}`);
            player.runCommandAsync(`title @s title §a§l任务完成！`);
            player.runCommandAsync(`playsound x.wancheng @s ~ ~ ~ 0.5`);
            player.runCommandAsync(`say §a取得了成就 §d${getTitleTitle[string - 1]}`);
            player.addTag(getItems[string - 1]);
            break;
          }
          if(i>=35)
          {
            player.runCommandAsync(`title @s title §c§l任务未提交`);
            break;
          }
        }
      }
  
      if (response2.selection === 1) {
        formMain(player);
      }
    }
  });
};

const formAbout = (player) => {
  const form = new ActionFormData().title(getTitle).body(`§a任务书使用开源项目:myQuestAPI\n§e模组作者: §rMinecraft皓星辰\n§c源代码来自: @abcdave\n§c特别鸣谢@abcdave的代码思路\n§d致谢名单：\n§aB站UP主：任桐雨，山海_夜兮的长视频录制\n§b爱发电用户：月亮，肝肝游戏Aifeita，兴千化，dotck的支持\n§d制作团队：冬之纪行诗团队 的技术性疑难解答`).button(getButtonBack);

  form.show(player).then(response => {
    if (response.selection === 0) {
      formMain(player);
    }
  });
};
